# Example that shows how to analyse PFA (reconstructed particles) using  LCIO format
# This example makes invariant mass of Higgs from 4 leptons. use H->ZZ* -> 4L sample 
#  Get 20 files in 2 threads as (see HepSim manual):
#  > hs-get gev250ee_pythia6_higgs_zz_4l%rfull001 gev250ee_pythia6_higgs_zz_4l 2 10  
# S.Chekanov (ANL)

from org.lcsim.event import *
from org.lcsim.util import *
from hep.physics.vec import *
from java.util import *
from java.io import *
from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import *
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle 
from math import *
from java.awt import *


# make list of files..
import glob
files=glob.glob("gev250ee_pythia6_higgs_zz_4l/*.slcio")
factory = LCFactory.getInstance()
reader = factory.createLCReader()
reader.open(files)


# histogram with Higgs and Z
h1=H1D("M(4l)",120,110,140)

# fit range
xmin=120
xmax=130


nEvent=0
while(1):
     evt=reader.readNextEvent()
     if (evt == None): break
     nEvent=nEvent+1
     if (nEvent%100==0): print nEvent, " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
     strVec = evt.getCollectionNames()
     if nEvent == 1:
            for col in  strVec:
                           print col           
     col = evt.getCollection("PandoraPFOCollection")
     nPFA = col.getNumberOfElements()

     electrons=[]
     muons=[]
     for i in range(nPFA):
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/lcio/SIOReconstructedParticle.html
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/event/ReconstructedParticle.html
          pa=col.getElementAt(i)
          charge=pa.getCharge()
          p4=pa.getMomentum()  
          ee=pa.getEnergy() 
          typep=pa.getType() # type
          p=LParticle(p4[0],p4[1],p4[2],ee)
          p.setCharge(charge)
          if (abs(typep)==11 and p.perp()>10):  electrons.append(p) # reconstructed electrons 
          if (abs(typep)==13 and p.perp()>10):  muons.append(p)     # reconstructed  muons 

#     print "Muons=",muons
#     print "Electrons=",electrons
     allp=electrons+muons
     if (len(electrons)==4 or len(muons)==4 or (len(electrons)==2 and len(muons)==2)):
               allp[0].add(allp[1])
               allp[0].add(allp[2])
               allp[0].add(allp[3])
               mass=allp[0].calcMass()
               h1.fill(mass)


# close and go to next
reader.close();






c1=HPlot()
c1.visible()
c1.setMarginLeft(90)
c1.setNameX("M (4l) [GeV]")
c1.setNameY("Events")
c1.setRange(xmin,xmax,0,100)

f=HFitter("chi2")   # build fitter 
f.setFunc("g")      # gaussian  
func=f.getFunc()
#print func.parameterNames()
f.setPar('mean',125); f.setPar('amplitude',10); f.setPar('sigma',0.5);
f.setRange(115,135) # fit range 
f.fit(h1)
fresult=f.getFittedFunc() # return final function 
res=f.getResult()
fPars  = res.fittedParameters()
fPars     = res.fittedParameters()
fParErrs  = res.errors()
fParNames = res.fittedParameterNames()
print "Fit results:"
for i in range(fresult.numberOfParameters()):
   print "par=",i,"=", fParNames[i]+" : "+str(fPars[i])+" +- "+str(fParErrs[i])

# make F1D function from IFunction in the same range as histogram
f2 = F1D("Gaussian",fresult,xmin,xmax)
f2.setColor(Color.blue)
f2.setPenWidth(3);

p1=P1D(h1)   # convert histogram to P1D array
# p1.doc()   # show its methods
c1.draw(p1)
c1.draw(f2) # draw fit line  

q1="mean=%4.3f &plusmn; %4.3f GeV"%(fPars[1],fParErrs[1])
q2="&sigma;=%4.3f &plusmn; %4.3f GeV"%(fPars[2],fParErrs[2])
l1=HLabel(q1,0.65,0.75, "NDC")
l1.setColor(Color.blue)
l1.setFont(Font("Helvetica", Font.PLAIN, 14))
c1.add(l1)
l2=HLabel(q2,0.7,0.7, "NDC")
l2.setColor(Color.blue)
l2.setFont(Font("Helvetica", Font.PLAIN, 14))
c1.add(l2)

c1.export("mc_higgs_zz.pdf") 
 
