# Example that shows how to make jets from PFA (reconstructed particles) using  LCIO format
# This example looks at jet resolution.
# This is a simple example that uses the Durham algorithm with pT>20 GeV
# It also does a Gaussian fit. 
# Use Z to bbar data 
#  Get 4 file in 2 thread (see HepSim manual):
#  > hs-get gev250ee_pythia6_qcd_all%rfull001 gev250ee_pythia6_qcd_all 2 4 
#
# author: S.Chekanov (ANL)


from org.lcsim.event import *
from org.lcsim.util import *
from hep.physics.vec import BasicHepLorentzVector,HepLorentzVector 
from java.util import *
from java.io import *
from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import *
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle 
from hep.physics.jet import FixNumberOfJetsFinder
from java.awt import *
import math 

# make list of files..
import glob
files=glob.glob("gev250ee_pythia6_qcd_all/*.slcio")
factory = LCFactory.getInstance()
reader = factory.createLCReader()
reader.open(files)

xmin=0.8
xmax=1.2
h1=H1D("20<pT<40",100,xmin,xmax)
h2=H1D("90<pT<110",100,xmin,xmax)
# h1.doc()  # look at API

# jet description
# http://java.freehep.org/freehep-physics/apidocs/hep/physics/jet/package-summary.html
# this is Jade
from hep.physics.jet import DurhamJetFinder,FixNumberOfJetsFinder
# fjet=FixNumberOfJetsFinder(2) # request 2 jets. The Jade algorithm at work 
fjet=DurhamJetFinder(0.05)

nEvent=0
while(1):
     evt=reader.readNextEvent()
     if (evt == None): break
     nEvent=nEvent+1
     # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
     if (nEvent%50==0): print "# Event: ",nEvent
     strVec = evt.getCollectionNames()
     if nEvent == 1:
            for col in  strVec:
                           print col           
     col = evt.getCollection("PandoraPFOCollection")
     nPFA = col.getNumberOfElements()
     # if (nEvent>2000): break

     alljets=[] # make a new list with jets 
     particles=ArrayList() # list of particles  
     for i in range(nPFA):
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/lcio/SIOReconstructedParticle.html
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/event/ReconstructedParticle.html
          pa=col.getElementAt(i)
          charge=pa.getCharge()
          p4=pa.getMomentum()  
          ee=pa.getEnergy() 
          typep=pa.getType() # type of PF 
          p=BasicHepLorentzVector(ee,p4[0],p4[1],p4[2])
          particles.add(p) # add particle to the list 
     #print particles.size()
     if (particles.size()>1): # ask for >1 particles
       fjet.setEvent(particles)
       # print "Nr of jets=", fjet.njets()  
       for i in range(fjet.njets()):
         # print "Jet=",i," Nr of particles in jet =",fjet.nParticlesPerJet(i) 
         pjet=fjet.jet(i)   # make  HepLorentzVector
         ee=pjet.t()  # magnitude
         p3=pjet.v3() # 3-vector  
         #print " ",ee,p3
         pl=LParticle(p3.x(),p3.y(),p3.z(),ee) # convert to a class with convinient kinematic methods  
         if (pl.perp()>20): alljets.append(pl)  

     ##### truth jets ########
     col = evt.getCollection("MCParticle")
     nMc=col.getNumberOfElements()
     nn=0
     particle_truth=ArrayList() # list of particles
     for i in range(nMc): # loop over all particles 
         par=col.getElementAt(i)
         if(par.getGeneratorStatus() == 1):
              vertex = par.getVertex();
              pdg=par.getPDG()
              momentum = par.getMomentum()
              ee=par.getEnergy()
              mass=par.getMass()
              px=momentum[0]
              py=momentum[1]
              pz=momentum[2]
              charge=par.getCharge()
              p=BasicHepLorentzVector(ee,px,py,pz)
              particle_truth.add(p)
              #print nn, "pdg=",pdg, " ",px,py,pz, " charge=",charge 
              nn=nn+1
     if (particle_truth.size()>1): # ask for >1 particles
       fjet.setEvent(particle_truth)
       alltruthjets=[] # make a new list with jets 
       for i in range(fjet.njets()):
         # print "Jet=",i," Nr of particles in jet =",fjet.nParticlesPerJet(i) 
         pjet=fjet.jet(i)   # make  HepLorentzVector
         ee=pjet.t()  # magnitude
         p3=pjet.v3() # 3-vector  
         #print " ",ee,p3
         pl=LParticle(p3.x(),p3.y(),p3.z(),ee) # convert to a class with convinient kinematic methods  
         if (pl.perp()>5): alltruthjets.append(pl)

       # match reco with truth  
       for i1 in range(len(alljets)):
          p1=alljets[i1]
          pt=p1.perp()
          phi=p1.phi()
          rap=p1.rapidity() 
          for i2 in range(len(alltruthjets)):
            p2=alltruthjets[i2]
            t_pt=p2.perp()
            t_phi=p2.phi()
            t_rap=p2.rapidity()
            dphi=(t_phi-phi)
            if (dphi>math.pi): dphi=2*math.pi-dphi 
            deta=(t_rap-rap)
            dd=math.sqrt(dphi*dphi+deta*deta) 
            if (dd<0.05):
                        rr=pt/t_pt  
                        if(t_pt>20 and t_pt<40):  h1.fill(rr)
                        if(t_pt>90 and t_pt<110): h2.fill(rr)

reader.close();

c1=HPlot("resolution",600,500,1,2)
# c1.doc() # look at API
c1.visible()
c1.setMarginLeft(80)
c1.setNameX("pT(jet, PFA) / pT(jet, truth)")
c1.setNameY("Events")
c1.setAutoRange()
c1.setRange(xmin,xmax,0,200)


############# fit first plot
f=HFitter("chi2")   # build fitter 
f.setFunc("g")      # gaussian + constant  
func=f.getFunc()
#print func.parameterNames()
f.setPar('mean',1); f.setPar('amplitude',100)
f.setPar('sigma',0.1)
f.setRange(xmin,xmax) # fit range 
f.fit(h1)
fresult=f.getFittedFunc() # return final function 
res=f.getResult()
fPars  = res.fittedParameters()
fPars     = res.fittedParameters()
fParErrs  = res.errors()
fParNames = res.fittedParameterNames()
print "Fit results:"
for i in range(fresult.numberOfParameters()):
   print "par=",i,"=", fParNames[i]+" : "+str(fPars[i])+" +- "+str(fParErrs[i])
p1=P1D(h1)   # convert histogram to P1D array
c1.draw(p1)
# make F1D function from IFunction in the same range as histogram
f2 = F1D("Gauss fit",fresult,xmin,xmax)
f2.setColor(Color.blue)
f2.setPenWidth(2);
c1.draw(f2) # draw fit line  
q1="mean=%4.3f &plusmn; %4.3f GeV"%(fPars[1],fParErrs[1])
q2="&sigma;=%4.3f &plusmn; %4.3f GeV"%(fPars[2],fParErrs[2])
l1=HLabel(q1,0.7,0.75, "NDC")
l1.setColor(Color.blue)
l1.setFont(Font("Helvetica", Font.PLAIN, 14))
c1.add(l1)
l2=HLabel(q2,0.7,0.7, "NDC")
l2.setColor(Color.blue)
l2.setFont(Font("Helvetica", Font.PLAIN, 14))
c1.add(l2)






c1.cd(1,2)
c1.setMarginLeft(80)
c1.setNameX("pT(jet, PFA) / pT(jet, truth)")
c1.setNameY("Events")
c1.setAutoRange()
c1.setRange(xmin,xmax,0,200)

f=HFitter("chi2")   # build fitter 
f.setFunc("g")      # gaussian + constant  
func=f.getFunc()
#print func.parameterNames()
f.setPar('mean',1); f.setPar('amplitude',100)
f.setPar('sigma',0.1)
f.setRange(xmin,xmax) # fit range 
f.fit(h2)
fresult=f.getFittedFunc() # return final function 
res=f.getResult()
fPars  = res.fittedParameters()
fPars     = res.fittedParameters()
fParErrs  = res.errors()
fParNames = res.fittedParameterNames()
print "Fit results:"
for i in range(fresult.numberOfParameters()):
   print "par=",i,"=", fParNames[i]+" : "+str(fPars[i])+" +- "+str(fParErrs[i])
p2=P1D(h2)   # convert histogram to P1D array
c1.draw(p2)
# make F1D function from IFunction in the same range as histogram
f2 = F1D("Gauss fit",fresult,xmin,xmax)
f2.setColor(Color.blue)
f2.setPenWidth(2);
c1.draw(f2) # draw fit line  
q1="mean=%4.3f &plusmn; %4.3f GeV"%(fPars[1],fParErrs[1])
q2="&sigma;=%4.3f &plusmn; %4.3f GeV"%(fPars[2],fParErrs[2])
l1=HLabel(q1,0.7,0.75, "NDC")
l1.setColor(Color.blue)
l1.setFont(Font("Helvetica", Font.PLAIN, 14))
c1.add(l1)
l2=HLabel(q2,0.7,0.7, "NDC")
l2.setColor(Color.blue)
l2.setFont(Font("Helvetica", Font.PLAIN, 14))
c1.add(l2)


c1.export("mc_jets_resol.pdf")
