# Example that shows how to analyse PFA (reconstructed particles) using  LCIO format
# This example makes invariant mass of 2 reconstructed electrons from Z-> e+e- and
# make a simple fit with a Gaussian. 
#  Get 4 files in 2 threads as (see HepSim manual):
#  > hs-get gev250ee_pythia6_zpole_ee%rfull001 gev250ee_pythia6_zpole_ee 2 4  
#
# author: S.Chekanov (ANL)

from org.lcsim.event import *
from org.lcsim.util import *
from hep.physics.vec import *
from java.util import *
from java.io import *
from org.lcsim.lcio import LCIOReader
from hep.io.sio import SIOReader
from hep.lcio.implementation.sio import SIOLCReader
from hep.lcio.implementation.io import LCFactory
from hep.lcio.event import *
from hep.lcio.io import *
from jhplot import *  # import graphics
from hephysics.particle import LParticle 
from math import *
from java.awt import Color

# make list of files..
import glob
files=glob.glob("gev250ee_pythia6_zpole_ee/*.slcio")
factory = LCFactory.getInstance()
reader = factory.createLCReader()
reader.open(files)
h1=H1D("Mass",70,50,120)
# h1.doc() # look at API


nEvent=0
while(1):
     evt=reader.readNextEvent()
     if (evt == None): break
     nEvent=nEvent+1
     # print " file event: ",evt.getEventNumber(), " run=",evt.getRunNumber()
     if (nEvent%100==0): print "# Event: ",nEvent
     strVec = evt.getCollectionNames()
     if nEvent == 1:
            for col in  strVec:
                           print col           
     col = evt.getCollection("PandoraPFOCollection")
     nPFA = col.getNumberOfElements()

     particles=[]
     for i in range(nPFA):
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/lcio/SIOReconstructedParticle.html
          # http://www.lcsim.org/sites/lcsim/apidocs/org/lcsim/event/ReconstructedParticle.html
          pa=col.getElementAt(i) # Represents a fully reconstructed particle.  
          #particles_PFA=pfo.getParticles() 
          #clusters=pfo.getClusters()
          #tracks=pfo.getTracks()
          charge=pa.getCharge()
          p4=pa.getMomentum()  
          ee=pa.getEnergy() 
          typep=pa.getType() # type
          p=LParticle(p4[0],p4[1],p4[2],ee)
          p.setCharge(charge)
          if (abs(typep)==11): particles.append(p) # take reconstructed electrons
     if (len(particles)>1):
       for i1 in range(0,len(particles)-1):
          p1=particles[i1]
          charge1=p1.getCharge()
          for i2 in range(i1+1,len(particles)):
              p2=particles[i2]
              charge2=p2.getCharge()
              p1.add(p2)
              if (charge1*charge2<0):
                  # print p1.calcMass()
                  mass=p1.calcMass()
                  h1.fill(mass)

reader.close();

c1=HPlot()
# c1.doc() # look at API
c1.visible()
c1.setMarginLeft(100)
c1.setNameX("Mass [GeV]")
c1.setNameY("Events")
c1.setRange(60,120,0,200)
c1.draw(h1)
c1.drawStatBox(h1)

xmin=85 
xmax=95  

f=HFitter("chi2")   # build fitter 
f.setFunc("g")      # gaussian + constant  
func=f.getFunc()
#print func.parameterNames()
f.setPar('mean',90); f.setPar('amplitude',100)
f.setRange(xmin,xmax) # fit range 
f.fit(h1)
fresult=f.getFittedFunc() # return final function 
res=f.getResult()
fPars  = res.fittedParameters()
fPars     = res.fittedParameters()
fParErrs  = res.errors()
fParNames = res.fittedParameterNames()
print "Fit results:"
for i in range(fresult.numberOfParameters()):
   print "par=",i,"=", fParNames[i]+" : "+str(fPars[i])+" +- "+str(fParErrs[i])

# make F1D function from IFunction in the same range as histogram
f2 = F1D("Gaussian",fresult,xmin,xmax)
f2.setColor(Color.blue)
f2.setPenWidth(3);

p1=P1D(h1)   # convert histogram to P1D array
# p1.doc()   # show its methods
c1.draw(p1)
c1.draw(f2) # draw fit line  

c1.export("mc_pflow_fit.pdf") 
 
